#include "settings.h"
#include "system.h"


/* Will use the macros created to pull for the done condition.  The CPU calculations for the next frame buffer
 * allowing for better performance (parallelism).
 */ 
#pragma altera_accelerate enable_interrupt_for_function hw_mandelbrot_engine_0
#pragma altera_accelerate enable_interrupt_for_function hw_mandelbrot_engine_1
#pragma altera_accelerate enable_interrupt_for_function hw_frame_stuffer



/* The Nios II data master connects to the first port of this onchip memory.  This hardware accelerator connects
 * to the second port to minimize the amount of arbitration.
 */  
#pragma altera_accelerate connect_variable hw_frame_stuffer/colour_lookup_table to colour_lookup_table/s2

/* Since in the ideal case this algorithm will write 8 back to back pixels to memory.  A single Avalon-MM master is
 * created for this connection which will be shared between all the pixel values.  This memory is shared by the
 * Nios II data master (r/w) and the VGA controller (r).
 */
#pragma altera_accelerate connect_variable hw_frame_stuffer/frame_buffer to ddr_sdram



/* This macro will allow for the number of engines to be expanded easier.  This algorithm is optimized
 * to work on one dimension instead of two (rows and columns).  Using only one dimension, a loop for
 * the rows is avoided which would have had an inital latency penalty each time it was reached (# of columns).
 * If the number of mandelbrot engines changes it's important to modify the file "settings.h" since this
 * calculation engine depends on the value specified in it.
 */
#define MANDELBROT_FRAME(current_x, current_y, step_dim, max_iter, fifo_ptr) \
{ \
  long long current_x_copy = (current_x); \
  long long current_y_copy = (current_y); \
  long long leftmost_x = (current_x); \
  long i, iters, column = 0; \
  long long ci, cr; \
  long long xsqr, ysqr; \
  long x, y; \
  for(i = 0; i < PIX_MAP_LENGTH; i+=NUM_OF_ENGINES) \
  { \
    iters=0; \
    x = 0; \
    y = 0; \
    xsqr = 0; \
    ysqr = 0; \
    ci = (current_y_copy)<<28; \
    cr = (current_x_copy)<<28; \
    while( ((xsqr + ysqr) < 0x0400000000000000LL) && (iters < (max_iter)) ) \
    { \
      xsqr = ((long long)x) * ((long long)x); \
      ysqr = ((long long)y) * ((long long)y); \
      y = (long)(((2 * ((long long)x) * ((long long)y)) + ci) >> 28); \
      x = (long)(((xsqr - ysqr) + cr) >> 28); \
      iters = iters + 1; \
    } \
    *fifo_ptr = iters; \
    current_x_copy = (column == (PIX_MAP_WIDTH - NUM_OF_ENGINES))? leftmost_x : current_x_copy + ((step_dim) * NUM_OF_ENGINES); \
    current_y_copy -= (column == (PIX_MAP_WIDTH - NUM_OF_ENGINES))? (step_dim) : 0; \
    column = (column == (PIX_MAP_WIDTH - NUM_OF_ENGINES))? 0 : column + NUM_OF_ENGINES; \
  } \
}

/* These FIFOs are not connected to Nios II so we can't use system.h definitions.  Be sure to modify
 * these values if the FIFOs are moved to different addresses.
 */
#define FIFO_0_S1_BASE 0x02000000
#define FIFO_0_S2_BASE 0x02000000
#define FIFO_1_S1_BASE 0x02000000
#define FIFO_1_S2_BASE 0x02000000



/* Each of these FIFO write pointers will be passed to the Mandelbrot macro.  There is only one
 * connection between the accelerator and the FIFO write port making this arbitration-less (fast)
 */
#pragma altera_accelerate connect_variable hw_mandelbrot_engine_0/fifo_0_write_ptr to fifo_0/in
#pragma altera_accelerate connect_variable hw_mandelbrot_engine_1/fifo_1_write_ptr to fifo_1/in




/* Each of these FIFO read pointers will be used by the hw_draw_int_mandelbrot.  There is only one
 * connection between the accelerator and the FIFO read port making this arbitration-less (fast)
 */
#pragma altera_accelerate connect_variable hw_frame_stuffer/fifo_0_read_ptr to fifo_0/out
#pragma altera_accelerate connect_variable hw_frame_stuffer/fifo_1_read_ptr to fifo_1/out





/* This is the hardware accelerator equivalent of "sw_draw_int_mandelbrot".  Accelerate the functions below without
 * cache flushing turned on.  Since there is no data shared between Nios II and these hardware accelerators,
 * disabling the data cache flushing is safe and leads to performance gains.
 * 
 * The functions you need to accelerate are as follows:
 * 
 * hw_mandelbrot_engine_x (all of them where x is 0, 1, 2, etc.... this calculates the pixel value)
 * hw_frame_stuffer (this take the pixel value and maps it to a colour, afterwards it stuff the data into DDR SDRAM)
 */


  /* This is where the majority of the accelerator work starts.  Each mandelbrot macro transverses the entire
   * frame, skipping 'n' pixels. 'n' is the number of engines so it must be a multiple of 2 and evenly divide
   * the frame width. Since each mandelbrot engine operates on side by side pixels, the starting x coordinate
   * must be shifted by the step dimension (represents the amount of space between pixels).  So the second engine
   * is ofset by "step_dim", the third engine by "step_dim * 2", the forth engine by "step_dim * 3", etc...
   * 
   * 
   * If each mandelbrot engine provides the pixel output 'p' where p is between 1 and the number of engines present
   * then the following grid represents the workload by each engine:
   * 
   * 1   2   3   4   5   6   7   8   1   2   3   4   5 .... etc
   * 1   2   3   4   5   6   7   8   1   2   3   4   5 .... etc
   * .... etc
   * 1   2   3   4   5   6   7   8   1   2   3   4   5 .... etc
   * 
   * So if a 640x480 image is to be calculated and eight engines are present, then each engine handles 80 pixels per
   * line. The engines operate on side by side pixels since the amount of calculation work is best divided by using
   * this approach. To understand this better observe the black areas of the image (known as the mandelbrot set).  These
   * are the locations where the maximum number of iterations occur. If the engines worked on alternate frames or large
   * sections of the image, not all of the engines share an equal work load. By balancing the workload the efficiency
   * of this hardware accelerator is maximized. 
   */



void hw_mandelbrot_engine_0(long long current_x, long long current_y, long long step_dim, int max_iter)
{
  volatile long * fifo_0_write_ptr = (long *)FIFO_0_S1_BASE;
  MANDELBROT_FRAME(current_x, current_y, step_dim, max_iter, fifo_0_write_ptr);  
}


void hw_mandelbrot_engine_1(long long current_x, long long current_y, long long step_dim, int max_iter)
{
  volatile long * fifo_1_write_ptr = (long *)FIFO_1_S1_BASE;
  MANDELBROT_FRAME(current_x, current_y, step_dim, max_iter, fifo_1_write_ptr);  
}



void hw_frame_stuffer(int max_iter, unsigned long * __restrict__ frame_buffer)
{
  /* Only one array accesses this colour table port so it's safe to use the restrict keyword.
   */
  long * __restrict__ colour_lookup_table = (long *)COLOUR_LOOKUP_TABLE_BASE;
  long pixel, iters0, iters1;

  volatile long * fifo_0_read_ptr = (long *)FIFO_0_S2_BASE;
  volatile long * fifo_1_read_ptr = (long *)FIFO_1_S2_BASE;

  

  /* This loop pulls the number of mandelbot iterations out of each dedicated pixel FIFO.  This loop if
   * running at full effectiveness reads all eight FIFOs in parallel, and sequentially writes eight times
   * to the frame buffer.  The mandelbrot engines take much more time than this so this loop does not hinder
   * the processing performance
   */
  for(pixel = 0; pixel < PIX_MAP_LENGTH; pixel+=NUM_OF_ENGINES)
  {
    /* read the pixels from all the FIFOs
     */
    iters0 = *fifo_0_read_ptr;
    iters1 = *fifo_1_read_ptr;


    /* sequentially write the mapped colour to the frame buffer for the video controller to read and display.
     */
    *frame_buffer++ = (iters0 == max_iter)? SET_COLOUR : colour_lookup_table[iters0 & COLOUR_MASK];
    *frame_buffer++ = (iters1 == max_iter)? SET_COLOUR : colour_lookup_table[iters1 & COLOUR_MASK];
  }    
}


/* Do not accelerate hw_draw_int_mandelbrot.  This function will be called in software
 * which in turn will call out all the accelerators.  If more mandelbrot engines are added
 * then add them to this function.  None of the functions called from here are blocking
 * and the done status will be checked from main.c, do_fluid_move.c, and do_fluid_zoom.c
 */
void hw_draw_int_mandelbrot(long long current_x, long long current_y, long long step_dim, int max_iter, unsigned long *frame_buffer)
{
  hw_mandelbrot_engine_0(current_x, current_y, step_dim, max_iter);
  hw_mandelbrot_engine_1(current_x + step_dim, current_y, step_dim, max_iter);
  hw_frame_stuffer(max_iter, frame_buffer);
}

/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2007 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
