/******************************************************************************
* Copyright  2007 Altera Corporation, San Jose, California, USA.             *
* All rights reserved. All use of this software and documentation is          *
* subject to the License Agreement located at the end of this file below.     *
*******************************************************************************
 * C2H Mandelbrot Design
 * 
 * Authors:
 * 
 *   RF - Initial Mandelbrot design
 * JCJB - Porting of the fluid demo of the initial design and C2H optimizations
 * 
 * Date:  02/25/2007
 * 
 * Revision History:
 * 
 * 0 - Inital design release
 * 
 * 
 * Design Instructions:
 * 
 * 1)  Connect video card (Lancelot) to proto2 header
 * 2)  Open "hw_mandelbrot.c" and highlight the hw_mandelbrot_engine_0 function
 * 3)  Accelerate the function with the following settings
 *       - Build software, generate SOPC Builder system, and run Quartus II compilation
 *       - Use hardware accelerator in place of software implementation
 * 4)  Repeat step 2 and 3 for all remaining functions
 *       - hw_mandelbrot_engine_2
 *       - ....
 *       - hw_mandelbrot_engine_n (accelerate all of the engine functions)
 *       - hw_frame_stuffer
 * 5)  Wait for the system generaton and compilation to complete (may take over half an hour)
 * 6)  Configure the device and download the compiled software
 * 
 * 
 * Usage:
 * 
 * SW0 = Hardware acceleration enabled
 * SW1 = Software only
 * SW2 = Next zoomout colour palette changes
 * SW3 = Pause the demo (press again to continue
 * 
 * 
 * Note:  If you don't have a video card you can still run the design.  To see the
 * maximum performance open the file "settings.h" and set "DISPLAY_PRESENT" to 0.
 * This will eliminate the triple buffer switching and prevent the video controller
 * from accessing the frame buffer.
 */



#include "settings.h"
#include "system.h"
#include "display_drivers.h"
#include "io.h"
#include "movement_functions.h"
#include "buttons.h"
#include "benchmark.h"
#include "alt_c2h_hw_frame_stuffer_irq.h"
#include "stdio.h"


 
inline void colour_swap();
void do_fluid_continuous_demo();


int main()
{
  printf("C2H Mandelbrot Design\n");
  printf("Press SW0 for hardware accelerated mode (default)\n");
  printf("Press SW1 for software only mode\n");
     
  colour_swap();
  
  // register the ISR that will handle swapping between software and hardware mandlebrot implementations
  init_button_pio();
  
  // allocate and clear frame buffers
  clear_frame_buffers();
  
  // prime the display controller(s) and start them (if display is present) 
  #if(DISPLAY_PRESENT == 1)  
    start_controller(VGA_CONTROLLER_BASE);
  #else
    stop_controller(VGA_CONTROLLER_BASE);  // just in case you switch from VGA support to none we need to make sure the controller stops
  #endif

  // prime the benchmark timer (timestamp timer)
  initialize_benchmark();

  // start up the demo
  do_fluid_continuous_demo(); 
      
  return 1;   
}


/* data structure to make it easier to get at the data contained in "demo_coordinates.h" */
struct coordinates {
  double x;
  double y;
  double dim_x;
  int max_iterations;
};

// This routine is the loop overhead for the continuous demo.
// The coordinates below are some of the interesting points in the Mandelbrot set.
void do_fluid_continuous_demo()
{
  const struct coordinates demo_locations[NUMBER_OF_COORDINATES] = {
    #include "demo_coordinates.h"
  };
  double previous_x = 0.25;
  double previous_y = 0.0;
  double next_x, next_y, next_dim_x;
  int i, next_maxiterations;


  while(1)
  {
    for(i = 0; i < NUMBER_OF_COORDINATES; i++)
    {
      /* read in the contents for the next zoom */
      next_x = demo_locations[i].x;
      next_y = demo_locations[i].y;
      next_dim_x = demo_locations[i].dim_x;
      next_maxiterations = demo_locations[i].max_iterations;
      
      do_fluid_move(previous_x, previous_y, next_x, next_y);
      do_fluid_zoom_in(next_x, next_y, next_dim_x, next_maxiterations);
      
      /* save off x and y to do the next move when the loop rolls around */
      previous_x = next_x;
      previous_y = next_y;
      colour_swap();  
    }
  }
}


/* This function is responsible for chaning the colour patterns after each
 * time the algorithm zooms out completely
 */
inline void colour_swap()
{
  int i;
  static int pattern = 0;
  unsigned long temp_colour;

  static unsigned long colours1[COLOUR_MASK + 1] = {
    #include "random_colours1.h"
  };
  
  static unsigned long colours2[COLOUR_MASK + 1] = {
    #include "random_colours2.h"
  };

  static unsigned long colours3[COLOUR_MASK + 1] = {
    #include "random_colours3.h"
  };
  
  static unsigned long colours4[COLOUR_MASK + 1] = {
    #include "random_colours4.h"
  };  
  
  static unsigned long colours5[COLOUR_MASK + 1] = {
    #include "random_colours5.h"
  };
  
  static unsigned long colours6[COLOUR_MASK + 1] = {
    #include "random_colours6.h"
  };

  
  // must wait for the accelerator to finish first (previous run)
  while(ACCELERATOR_MANDELBROT_HW_FRAME_STUFFER_MANAGED_INSTANCE_BUSY() == 1) {}
  
  /* if you add your own pattern just include it above and add to this if statement
   */
  if(return_colour_status())
  { 
    for(i = 0; i < 256; i++)
    {
      temp_colour = (pattern == 0)? colours1[i] :
                    (pattern == 1)? colours2[i] :
                    (pattern == 2)? colours3[i] :
                    (pattern == 3)? colours4[i] :
                    (pattern == 4)? colours5[i] : colours6[i];
      IOWR_32DIRECT(COLOUR_LOOKUP_TABLE_BASE, i*4, temp_colour); 
    }

    /* increase the modulo value if you add your own patterns
     */
    pattern = (pattern + 1) % 6;  // rotate through the colour patterns in a circle
  }
}

/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2007 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
