#include "io.h"
#include "system.h"
#include "settings.h"


/* There are three buffers "CURRENT", "NEXT", and "CALCULATE".  CURRENT represents
 * the buffer being displayed, NEXT represents the next buffer that has been rendered,
 * and CALCULATE represents the buffer currently be constructed (rendered)
 * 
 * We have DDR SDRAM all to ourselves for video buffering */
static unsigned long *VGA_FRAME_BUFFER_CURRENT_PTR = (unsigned long *)DDR_SDRAM_BASE;
static unsigned long *VGA_FRAME_BUFFER_NEXT_PTR = (unsigned long *)(DDR_SDRAM_BASE + (4 * PIX_MAP_HEIGHT * PIX_MAP_WIDTH));
static unsigned long *VGA_FRAME_BUFFER_CALCULATE_PTR = (unsigned long *)(DDR_SDRAM_BASE + (8 * PIX_MAP_HEIGHT * PIX_MAP_WIDTH));


void clear_frame_buffers()
{
  unsigned long i;
  
  for(i = 0; i < (PIX_MAP_HEIGHT * PIX_MAP_WIDTH); i++)
  {
    IOWR_32DIRECT((unsigned long)VGA_FRAME_BUFFER_CURRENT_PTR, 4*i, 0);
    IOWR_32DIRECT((unsigned long)VGA_FRAME_BUFFER_NEXT_PTR, 4*i, 0);
    IOWR_32DIRECT((unsigned long)VGA_FRAME_BUFFER_CALCULATE_PTR, 4*i, 0);

  }
}

void start_controller(unsigned long base)
{
  IOWR(base, 1, (unsigned long)VGA_FRAME_BUFFER_CURRENT_PTR);
  IOWR(base, 2, PIX_MAP_WIDTH*PIX_MAP_HEIGHT*4);  // buffer length in bytes
  IOWR(base, 0, 1);  // go
}


void stop_controller(unsigned long base)
{
  IOWR(base, 0, 0); // stop  
}


void rotate_frame_buffers(unsigned long base)
{
  unsigned long * temp =  VGA_FRAME_BUFFER_CURRENT_PTR;
    
  /* if the display controller is still outputting two frames behind
   * then we need to block the rotation until the controller catches
   * up to the previously rendered frame.  The rotation is as follows:
   *
   * CURRENT <------- NEXT <------- CALCULATE
   *    |                                ^
   *    |                                |
   *     ---------------------------------
   * 
   * At VGA offset 3 is the frame buffer location of the frame currently
   * being drawn to the screen.  If that value equals "NEXT" then the
   * controller is two frames behind so we must wait. 
   */
  while (IORD(base, 3) == (unsigned long)VGA_FRAME_BUFFER_NEXT_PTR) {}

  // write the new buffer location to the controller
  IOWR(base, 1, (unsigned long)VGA_FRAME_BUFFER_NEXT_PTR);
  
  // rotate the buffers via a three line switch
  VGA_FRAME_BUFFER_CURRENT_PTR = VGA_FRAME_BUFFER_NEXT_PTR;
  VGA_FRAME_BUFFER_NEXT_PTR = VGA_FRAME_BUFFER_CALCULATE_PTR;
  VGA_FRAME_BUFFER_CALCULATE_PTR = temp;
}


/* Accessor functions, this is the only way the outside world *SHOULD* have
 * access to the frame buffers */
unsigned long * return_calculation_buffer()
{
  return VGA_FRAME_BUFFER_CALCULATE_PTR;   
}

unsigned long * return_next_buffer()
{
  return VGA_FRAME_BUFFER_NEXT_PTR;   
}

unsigned long * return_current_buffer()
{
  return VGA_FRAME_BUFFER_CURRENT_PTR; 
}

/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2007 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
